<?php
/**
 * Clip Name: Ultimate Member Podcasts Clip
 * Clip URI: https://clipbank.krolyn.com/clip/um-member-prodcasts
 * Description: This Clip adds an optional podcast feed to member profiles in the Ultimate Member plugin.
 * Version: 1.0
 * Author: Krolyn Studios
 * Author URI: https://clipbank.krolyn.com
 * Required Plugins: ultimate-member
 */

class UM_Member_Podcasts_Clip {

	function __construct() {

		if( is_admin() ) {
			add_filter( 'um_settings_structure', [ $this, 'um_member_podcast_settings'], 15 );
		}
		add_filter( 'um_predefined_fields_hook', [ $this, 'um_add_member_podcast_fields' ] );
		add_filter( 'um_account_page_default_tabs_hook', [ $this, 'account_tab_in_um_podcasts' ], 100 );
		add_filter( 'um_account_content_hook_podcasts', [ $this, 'um_account_member_podcast_fields' ] );
		add_filter( 'um_profile_tabs', [ $this, 'um_profile_member_podcast_tab' ], 1000 );
		add_filter( 'um_user_profile_tabs', [ $this, 'um_display_member_podcast_tab' ], 2000 );
		add_action( 'um_profile_content_podcasts_default', [ $this, 'um_display_member_podcast_content' ] );
		add_action( 'um_account_pre_update_profile', [ $this, 'um_save_member_podcast_fields' ], 100 );
	}

	// Add settings
	public function um_member_podcast_settings( $settings ) {

		$key = ! empty( $settings['extensions']['sections'] ) ? 'member_podcast_section' : '';
		$number = UM()->options()->get( 'member_podcast_number' );
		$settings['extensions']['sections'][$key] = [
			'title'     => __( 'Member Podcasts', 'wp-clips' ),
			'fields'    => [
				[
					'id'       	=> 'member_podcast_roles',
					'type'     	=> 'select',
					'label'    	=> __( 'Roles to include member podcast option', 'wp-clips' ),
					'tooltip' 	=> __( 'Defaults to all roles.', 'wp-clips' ),
					'options' 	=> [ '' => __( 'None', 'wp-clips' ) ] + UM()->roles()->get_roles(),
					'multi'     => true
				],
				[
					'id'       	=> 'member_podcast_number',
					'type'     	=> 'number',
					'label'    	=> __( 'Number of podcast episodes to display', 'wp-clips' ),
					'tooltip' 	=> __( 'Maximum displayed is 25.', 'wp-clips' ),
					'value'		=> empty( $number ) ? 5 : ( $number <= 25 ? $number : 25 ),
					'default'	=> 5
				],
				[
					'id'       	=> 'member_podcast_hide_excerpt',
					'type'     	=> 'checkbox',
					'label'    	=> __( 'Hide podcast excerpts', 'wp-clips' ),
					'default'	=> 0
				]
			]
		];

		return $settings;
	}

	// Add predefined fields
	public function um_add_member_podcast_fields( $fields ) {

		$fields['podcast_feed'] = [
						'title' => __( 'Podcast Feed','wp-clips' ),
						'metakey' => 'podcast_feed',
						'type' => 'url',
						'label' => __( 'Podcast Feed','wp-clips' ),
						'help' => __( 'You must add your native podcast feed address to activate the feature','wp-clips' ),
						'placeholder' => __( 'URL (required)', 'wp-clips' ),
						'url_target' => '_blank',
						'url_rel' => 'nofollow',
						'account_only' => true,
					];
		$fields['podcast_itunes'] = [
						'title' => __('iTunes Channel','wp-clips'),
						'metakey' => 'podcast_itunes',
						'type' => 'url',
						'label' => __('iTunes Channel','wp-clips'),
						'help' => __( 'Add your iTunes address to display a link to your channel','wp-clips' ),
						'placeholder' => __( 'URL (optional)', 'wp-clips' ),
						'url_target' => '_blank',
						'url_rel' => 'nofollow',
						'account_only' => true,
					];
		$fields['podcast_google'] = [
						'title' => __('Google Play','wp-clips'),
						'metakey' => 'podcast_google',
						'type' => 'url',
						'label' => __('Google Play Channel','wp-clips'),
						'help' => __( 'Add your Google Play address to display a link to your channel','wp-clips' ),
						'placeholder' => __( 'URL (optional)', 'wp-clips' ),
						'url_target' => '_blank',
						'url_rel' => 'nofollow',
						'account_only' => true,
					];
		$fields['podcast_soundcloud'] = [
						'title' => __('SoundCloud','wp-clips'),
						'metakey' => 'podcast_soundcloud',
						'type' => 'url',
						'label' => __('SoundCloud Channel','wp-clips'),
						'help' => __( 'Add your SoundCloud address to display a link to your channel','wp-clips' ),
						'placeholder' => __( 'URL (optional)', 'wp-clips' ),
						'url_target' => '_blank',
						'url_rel' => 'nofollow',
						'account_only' => true,
					];
		return $fields;
	}

	// Get channel ID
	private function get_podcast_feed( $user_id ) {

		return get_user_meta( $user_id, 'podcast_feed', true );
	}

	// Check accepted roles for feature
	private function check_roles( $user_id ) {

		$user_role = UM()->user()->get_role();
		$acc_roles = UM()->options()->get( 'member_podcast_roles' );

		if( empty( $acc_roles ) || in_array( $user_role, $acc_roles ) ) return true;

		return false;
	}

	public function account_tab_in_um_podcasts( $tabs ) {

		$user_id = um_user('ID' );

		if( $this->check_roles( $user_id ) ) {

			$tabs[740]['podcasts']['icon'] = 'um-faicon-microphone';
			$tabs[740]['podcasts']['title'] = __( 'Podcast Settings', 'wp-clips' );
			$tabs[740]['podcasts']['custom'] = true;
			$tabs[740]['podcasts']['show_button'] = true;
			$tabs[740]['podcasts']['submit_title'] = __( 'Update Options', 'wp-clips' );
		}
		return $tabs;
	}

	// Add fields to podcast tab
	public function um_account_member_podcast_fields( $output ) {

		$user_id = um_user('ID' );

		if( $this->check_roles( $user_id ) ) {

			$output .= sprintf( '<p>%s.</p>', __( 'A podcast feed is required to activate the feature', 'wp-clips' ) );

			$args = implode( ',', [ 'podcast_feed','podcast_itunes','podcast_google','podcast_soundcloud' ] );
			$fields = UM()->builtin()->get_specific_fields( $args );
			$fields = apply_filters( 'um_account_secure_fields', $fields, $user_id );

			foreach( $fields as $key => $data ) {
				$output .= UM()->fields()->edit_field( $key, $data );
			}
		}
		return $output;
	}

	// Save fields in account tab
	public function um_save_member_podcast_fields() {

		$user_id = um_user('ID');

		foreach( [ 'podcast_feed','podcast_itunes','podcast_google','podcast_soundcloud' ] as $field ) {
			if( isset( $_POST[$field] ) ) {
				update_user_meta( $user_id, $field, $_POST[$field] );
			}
		}
	}

	// Add tab to profile
	public function um_profile_member_podcast_tab( $tabs ) {
		
		$tabs['podcasts'] = [
			'name' => __( 'Podcasts', 'wp-clips' ),
			'icon' => 'um-faicon-microphone',
			'custom' => true
		];

		if( ! isset( UM()->options()->options['profile_tab_' . 'podcasts'] ) ) {
			UM()->options()->options[ 'profile_tab_' . 'podcasts' ] = true;
		}

		return $tabs;
	}

	// Check if user has tab available
	public function um_display_member_podcast_tab( $tabs ) {

		$user_id = um_profile_id();

		if( ! $this->check_roles( $user_id ) || empty( $this->get_podcast_feed( $user_id ) ) ) {
			unset( $tabs['podcasts'] );
		}

		return $tabs;
	}

	// Add content to profile tab
	public function um_display_member_podcast_content( $args ) {

		$user_id = um_profile_id();
		$feedurl = $this->get_podcast_feed( $user_id );
		$feedurls = strpos( $feedurl, ',' ) !== false ? explode( ',', $feedurl ) : $feedurl;
	
		if( is_ssl() ) {

			foreach( $feedurls as $url ) {
				if( ! preg_match( '|^https ?://|', $url ) ) {
					$ssltest = $ssltest + 1;
					$nonssl .= $url. ", ";
				}
			}
			if( $ssltest > 0 ) {
				echo '<p>' . __( 'The podcast feed you entered is not secure. You must have a secure feed or else the audio files will not play. (Feed: ', 'wp-clips' ) . rtrim( $nonssl, ', ' ) . ')</p>';
			}
		}

		$feed = fetch_feed( $feedurls );
		$maxitems = 0;

		if( ! is_wp_error( $feed ) ) {

			$number = intval( UM()->options()->get( 'member_podcast_number' ) );
			$maxnumber = ! empty( $number ) && $number <= 25 ? $number : 25;
			$maxitems = $feed->get_item_quantity( $maxnumber ); 
			$feed_items = $feed->get_items( 0, $maxitems );
		}

		if( ! is_feed() ) {

			$itunes = get_user_meta( $user_id, 'podcast_itunes', true );
			$google = get_user_meta( $user_id, 'podcast_google', true );
			$soundcloud = get_user_meta( $user_id, 'podcast_soundcloud', true );
			$hide_excerpt = UM()->options()->get( 'member_podcast_hide_excerpt' );

			echo '<div class="um-podcast-subscribe"><span>' . __( 'Subscribe to:', 'wp-clips' ) . '</span>&nbsp;';
			echo '&nbsp;<a href="'. $feedurl . '" title="'. esc_attr( um_user( 'display_name' ) ) . ' Podcast"><i class="um-faicon-microphone"></i></a>';
			if( ! empty( $itunes ) ) {
				echo '&nbsp;<a href="'. $itunes . '" title="'. esc_attr( um_user( 'display_name' ) ) . ' on iTunes"><i class="um-faicon-apple"></i></a>';
			}
			if( ! empty( $google ) ) {
				echo '&nbsp;<a href="'. $google . '" title="'. esc_attr( um_user( 'display_name' ) ) . ' on Google Play"><i class="um-faicon-play"></i></a>';
			}
			if( ! empty( $soundcloud ) ) {
				echo '&nbsp;<a href="'. $soundcloud . '" title="'. esc_attr( um_user( 'display_name' ) ) . ' on SoundCloud"><i class="um-faicon-soundcloud"></i></a>';
			}
			echo '</div>';

			if ( $maxitems == 0 ) {     
				echo '<p>' . __( 'No items available.', 'wp-clips' ) . '</p>';
			}
			else { 

				echo '<div class="um-ajax-items">';
	
				foreach ( $feed_items as $item ) {
	
					$link = $item->get_permalink();
	
					if( empty( $hide_excerpt ) ) {
						
						$excerpt = $item->get_description();
						
						if( ! empty( $excerpt ) ) {
	
							$excerpt = strip_tags( str_replace( '</p>', ' ', $excerpt ) );
							$excerpt = explode( ' ', $excerpt, 50 );
							$key_last = intval( array_key_last( $excerpt ) );
							$excerpt[$key_last] = ' &hellip; ';
							if( ! empty( $link ) ) {
								$excerpt[$key_last] .= '<a href="'. $link . '" target="_blank">' . __( 'read more', 'wp-clips' ) . '</a>';
							}
							$excerpt = implode( ' ', $excerpt );
						}
					}
	
					if( $enclosure = $item->get_enclosure() ) {
						$audio = $enclosure->get_link();
						$audio = preg_replace('/\?.*/', '', $audio);
					}
	
					echo '<div class="um-item">';
					if( ! empty( $link ) ) {
						echo '<div class="um-item-link"><i class="um-faicon-microphone"></i><a href="'. $link . '" target="_blank">' . $item->get_title() . '</a></div>';
					}
					echo '<div class="um-item-meta"><span>' . $item->get_date("F j, Y | g:i a") . '</span></div>';
					if( ! empty( $excerpt ) ) {
						echo '<div class="um-item-desc"><p>' . $excerpt . '</p></div>';
					}
					echo '<div class="um-podcast-player">';
					$attr = [ 'src' => $audio, 'loop' => '', 'autoplay' => '', 'preload' => 'none' ]; 
					echo wp_audio_shortcode( $attr );
					echo '</div></div>';
				}

			echo '</div>';
		}

			add_action( 'wp_footer', function() {
				?>
				<script type="text/javascript">
					jQuery(document).ready(function($) {
						$( '.um-profile-body.podcasts .more' ).on( 'click', function() {
							$( this ).parent().next( '.summary' ).slideDown();
						});
					});
	
				</script><?php
			});
		}
	}
}

new UM_Member_Podcasts_Clip();